# OBJ - Framework to program bots
#
# obj/cmds.py
#
# Copyright 2019 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 01-01-2019 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" basic commands. """

__version__ = 1

## imports

from obj.base import Dotted, Selector, Store, cfg, get_cls, get_type
from obj.base import fn_time, match_types
from obj.base import homedir, slice, starttime, __version__, __txt__
from obj.fleet import Fleet
from obj.utils import days, elapsed, get_exception, kill, running
from obj.utils import set_completer
from obj.users import Users

import json
import logging
import obj
import os
import pkgutil
import sys
import time

## defines

class Aliases(Dotted):

    """ aliases table. """
    
    pass

class Log(Dotted):

    """ log item. """
    
    pass

## commands

def ed(event):
    """ edit objects. """
    if not event.args:
        event.reply("ed <%s>.name=value <name=value>" % "|".join(set([x.split(".")[-1].lower() for x in os.listdir(cfg.workdir)])))
        return
    store = Store()
    o = store.last(event.selector, event.type)
    if not o:
        return
    if not event.setter:
        event.reply(o)
        return
    for key, value in event.setter.items():
        val = None
        try:
            value = '"%s"' % value
            val = json.loads(value)
        except (TypeError, json.decoder.JSONDecodeError) as ex:
            event.reply("%s %s" % (str(ex), value))
            continue
        try:
            val = int(val)
        except:
            try:
                val = float(val)
            except:
                pass
        t = type(val)
        if t == str:
            o.set_attr(key, val)
        elif t == list:
            o.set_attr(key, val)
        elif val in ["True", "true"]:
            o.set_attr(key, True)
        elif val in ["False", "false"]:
            o.set_attr(key,  False)
        else:
            o[key] = val
    o.save()
    event.reply(o)

def deny(event):
    users = Users()
    user = users.get_user(event.args[0])
    user.perms.remove(event.args[1])
    user.save()
    event.reply("ok %s" % event.args[0])
    
def find(event):
    """ find objects. """
    if not event.args:
        event.reply("find %s" % "|".join(set([x.split(".")[-1].lower() for x in os.listdir(cfg.workdir)])))
        return
    store = Store()
    if len(event.args) == 1:
        l = store.last(event.selector)
        event.reply("find %s %s" % (event.args[0], "|".join(sorted([x for x in l.keys() if x and x[0] != "_"]))))
        return
    starttime = time.time()
    nr = -1
    for o in store.find(event.selector):
        nr += 1
        txt = ""
        if event.index != None and event.index != nr:
            continue
        if "d" in event.options:
            event.reply(str(o))
            continue
        if event.dkeys:
            txt = "%s %s" % (nr, o.format(event.selector._dkeys or event.dkeys))
        else:
            txt = "%s %s" % (nr, o.format(o.keys()))
        if "t" in event.options:
            txt += " " + days(o)
        event.reply(txt)
    event.reply("ok %s %s" % (nr+1, elapsed(time.time()-starttime)))

def load(event):
    """ load module. """
    fleet = Fleet()
    bot = fleet.get_bot(event.orig)
    if not event.args:
        res = []
        for nn in pkgutil.walk_packages(obj.__path__, "obj."):
            res.append(nn[1])
        event.reply("load %s" % "|".join(sorted(res)))
        return
    name = event.args[0]
    mod = bot.walk(name)
    set_completer(bot._names)
    if "start" in dir(mod):
        try:
            mod.start()
        except Exception as ex:
            logging.error(get_exception())
        event.reply("%s loaded" % name)

def log(event):
    """ log some text. """
    o = Log(txt=event.rest)
    o.save()
    event.ok(1)

def meet(event):
    """ add a new user. """
    origin, *perms = event.args
    users = Users()
    origin = users.userhosts.get(origin, origin)
    if users.meet(origin, perms):
        event.reply("%s added." % origin)

def restore(event):
    """ undelete objects.  """ 
    if not event.match:
        event.reply("missing selector.")
        return
    store = Store()
    starttime = time.time()
    nr = -1
    for type in match_types(event.type):
        for o in store.find(event.match, type, True):
            nr += 1
            if event.index != None and event.index != nr:
                continue
            o._deleted = False
            o.save()
    event.reply("ok %s %s" % (nr+1, elapsed(time.time()-starttime)))

def rm(event):
    """ set _deleted flag. """
    if not event.selector:
        event.reply("missing selector.")
        return
    store = Store()
    starttime = time.time()
    nr = -1
    for o in store.find(event.selector):
        nr += 1
        if event.index != None and event.index != nr:
            continue
        o._deleted = True
        o.save()
    event.reply("ok %s %s" % (nr+1, elapsed(time.time()-starttime)))

def show(event):
    """ show internals. """
    if not event.args:
        event.reply("show cfg|cmds|keys|license|me|mods|tasks|uptime|version")
        return
    fleet = Fleet()
    store = Store()
    args = []
    cmd, *args = event.args
    if cmd == "cfg":
        event.reply(cfg)
    elif cmd == "cmds":
        bot = fleet.get_bot(event.orig)
        if bot:
            event.reply(",".join(sorted(bot._names.keys())))
    elif cmd == "fleet":
        try:
            nr = int(args[1])
        except (IndexError, ValueError) as ex:
            event.reply(str([get_type(b) for b in fleet]))
            return
        try:
            event.reply(str(fleet.bots[nr]))
        except IndexError:
            pass
    elif cmd == "keys":
        if not args:
            event.reply("show keys %s" % "|".join(sorted(set([x.split(".")[-1].lower() for x in os.listdir(cfg.workdir)]))))
            return
        s = Selector(args[0])
        l = store.last(s)
        if l:
            event.reply("|".join([x for x in l.keys() if not x.startswith("_")]))
    elif cmd == "license":
        for txt in open(__file__).readlines()[:28]:
            event.reply(txt.strip())
    elif cmd == "mods":
        bot = fleet.get_bot(event.orig)
        event.reply("|".join(sorted(set(bot._names.values()))))
    elif cmd == "me":
        event.reply("email: bthate@dds.nl | irc: botfather on irc.freenode.net | pypi: https://pypi.org/project/obj | source: http://bitbucket.org/bthate/obj")
    elif cmd == "tasks":
        psformat = "%-8s %-60s"
        result = []
        try:
            getnr = int(event.args[1])
        except (ValueError, IndexError):
            getnr = None
        for thr in sorted(running(), key=lambda x: x.getName()):
            o = Dotted()
            o.update(vars(thr))
            if getattr(o, "sleep", None):
                up = o.sleep - int(time.time() - o._time.latest)
            else:
                up = int(time.time() - starttime)
            thrname = thr.getName()
            result.append((up, thrname, o))
        nr = -1
        for up, thrname, o in sorted(result, key=lambda x: x[0]):
            nr += 1
            if getnr != None and getnr == nr:
                event.reply(o)
                return
            res = "%s  %s" % (nr, psformat % (elapsed(up), thrname[:30]))
            event.reply(res.rstrip())
    elif cmd == "types":
        event.reply("|".join(os.listdir(cfg.workdir)))
    elif cmd == "uptime":
        event.reply(elapsed(time.time() - starttime))
    elif cmd == "version":
        event.reply("OBJ %s - %s" % (__version__, __txt__))

def stop(event):
    """ kill a task. """
    if not event.args:
        event.reply("stop <match>")
        return
    n = event.args[0]
    kill(n)
    event.reply("ok %s" % n)

def unload(event):
    """ unload module. """
    fleet = Fleet()
    bot = fleet.get_bot(event.orig)
    name = event.args[0]
    if name == "obj.cmds":
        event.reply("can't unload obj.cmds")
        return
    mod = bot._table.get(name, None)
    if mod and "stop" in dir(mod):
        try:
            mod.stop()
        except Exception as ex:
            logging.error(get_exception())
    try:
        bot.unload(name)
        set_completer(bot._names)
        event.reply("%s unloaded" % name)
    except KeyError:
        event.reply("no %s" % name)
