# OBJ - Framework to program bots
#
# obj/event.py
#
# Copyright 2019 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 01-01-2019 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    event class.
    
"""

__version__ = 1

## imports

from obj.base import Dotted, Selector, Store, cfg, match_types

import os
import threading

## classes

class Event(Dotted):

    """ parse a string into a command callback combo. """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._default = ""
        self._ready = threading.Event()
        self._result = []
        self._threaded = False
        self.cc = "!"

    def __getattr__(self, name):
        if name not in dir(self):
            self[name] = self._default
        return super().__getattr__(name)

    def get_cmd(self):
        v = self.get("txt", None)
        if v != None and v.strip():
            return v.strip().split()[0]
        return ""

    def ok(self, txt=""):
        self.reply("ok %s" % txt or self.get_cmd())
                    
    def parse(self):
        self._result = []
        self.args = []
        self.cmd = self.get_cmd()
        self.dkeys = self.dkeys or []
        self.index = None
        self.iscommand = False
        self.options = cfg.options
        self.selector = Selector(self.txt)
        self.txt = self.txt.strip()
        if self.txt and self.txt[0] == self.cc:
            self.txt = self.txt[1:]
            self.iscommand = True
        splt = self.txt.split()
        getopts = False
        nr = -1
        for arg in splt:
            nr += 1
            if nr == 0:
                self.cmd = arg
                continue
            if arg == "-o":
                getopts = True
                continue
            if getopts:
                getops = False
                self.options.extend(arg.split(","))
                continue
            if nr >= 1:
                self.dkeys.append(arg)
            self.args.append(arg)
        self.rest = " ".join(self.args)
        for c in self.options:
            try:
                self.index = int(c)
            except ValueError:
                pass

    def ready(self):
        self._ready.set()

    def reply(self, txt):
        try:
            txt = txt.pretty()
        except:
            pass
        self._result.append(txt)

    def show(self, bot=None):
        for txt in self._result:
            if not txt:
                continue
            bot.say(self.channel, txt)

    def wait(self):
        self._ready.wait()
