# OBJ - timestamped JSON objects
#
# obj/task.py
#
# Copyright 2019 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 01-01-2019 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    a OBJ thread, launch tasks, get a list of running tasks or kill a task.

"""

__version__ = 1

## imports

from obj.base import Dotted, starttime
from obj.fleet import Fleet
from obj.utils import get_exception, elapsed, name

import _thread
import logging
import obj
import queue
import threading
import time

## classes

class Task(threading.Thread):

    def __init__(self, *args, **kwargs):
        super().__init__(None, self.run, "", [], kwargs, daemon=False)
        self._queue = queue.Queue()
        self._func = None
        self._result = None
        self._stopped = False
         
    def __iter__(self):
        return self

    def __next__(self):
        for k in dir(self):
            yield k

    def put(self, func, *args, **kwargs):
        """ send workload to the task. """
        self._queue.put((func, args, kwargs))

    def run(self):
        """ task loop. """
        (func, args, kwargs) = self._queue.get()
        try:
            e = args[0]
            txt = e.txt
        except (IndexError, AttributeError):
            txt = name(func)
        self.setName(txt)
        starttime = time.time()
        self._result = func(*args, **kwargs)
        try:
            args[0]._elapsed = time.time() - starttime
        except:
            pass
        return self._result

    def join(self):
        """ join a task. """
        super().join()
        return self._result
