# OBJ - Framework to program bots
#
# obj/users.py
#
# Copyright 2019 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 01-01-2019 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    manages users.
    
"""

__version__ = 11

## imports

from obj.base import Dotted, Selector, Store
from obj.utils import stripped

import logging

## exceptions

class ENOORIGIN(Exception):

    """ origin is missing. """

    pass

class EUSER(Exception):

    """ user does not exist. """

    pass


## classes

class User(Dotted):

    """ User class to stored user data in. """

    pass

class Users(Store):

    """ Store class to access user objects. """

    userhosts = Dotted()

    def allowed(self, origin, perm):
        if not origin:
            raise ENOORIGIN
        perm = perm.upper()
        user = self.get_user(origin)
        if user:
            if perm in user.perms:
                logging.debug("allow %s %s" % (origin, perm))
                return True
        logging.error("denied %s %s" % (origin, perm))
        return False

    def delete(self, origin, perm):
        for user in self.get_users(origin):
            try:
                user.perms.remove(perm)
                user.save()
                return True
            except ValueError:
                pass
        return user

    def get_users(self, origin="", type=""):
        s = Selector()
        s._types.append(type or "obj.users.User")
        if origin:
            s.user = origin
        return self.find(s)

    def get_user(self, origin, type=""):
        s = Selector()
        s._types.append(type or "obj.users.User")
        if origin:
            s.user = origin
        return self.last(s)

    def meet(self, origin, perms=[]):
        logging.warn("meet %s" % origin)
        user = User()
        user.user = origin
        user.perms = ["USER", ]
        user.perms.extend(perms)
        user.save()
        return user

    def oper(self, origin):
        logging.warn("oper %s" % origin)
        user = self.get_user(origin)
        if not user:
            user = User()
            user.user = origin
            user.perms = ["OPER", "USER"]
            user.save()
        return user

    def perm(self, origin, permission):
        user = self.get_user(origin)
        if not user:
            raise EUSER(origin)
        if permission.upper() not in user.perms:
            user.perms.append(permission.upper())
            user.save()
        return user
