#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# OBJ - Framework to program bots
#
# setup.py
#
# Copyright 2019 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 01-01-2019 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" setup.py """

#from distutils.core import setup
from setuptools import setup

setup(
    name='obj',
    version='98',
    url='https://bitbucket.org/bthate/obj',
    author='Bart Thate',
    author_email='bthate@dds.nl',
    description="Framework to program bots",
    long_description="""
| pypi: https://pypi.org/project/obj | source: http://bitbucket.org/bthate/obj | email: bthate@dds.nl | botfather at #dunkbots/freenode

OBJ is a framework you can use to program bots, it's has it's own shell (the obj program) that has the following commands:

::

 ed                            - edit objects.
 find                          - find objects.
 load                          - load module.
 log                           - log some text.
 meet                          - add a new user.
 rm                            - set _deleted flag.
 show                          - show internals.
 unload                        - unload module.

the show command can be used to check status:

::

 cfg                           - show main config
 cmds                          - show available commands
 license                       - show license
 mods                          - show loaded modules
 tasks                         - show running tasks
 uptime                        - show uptime
 version                       - show version

the following modules are available in the OBJ package:

::

 obj.base                      - base classes.
 obj.bot                       - bot base class.
 obj.clock                     - timer, repeater.
 obj.cmds                      - basic commands.
 obj.dcc                       - direct client to client bot.
 obj.event                     - event class.
 obj.fleet                     - list of bots.
 obj.handler                   - queued event handler.
 obj.irc                       - irc bot.
 obj.loader                    - load modules into a table and scan for comands.
 obj.select                    - select based loop.
 obj.task                      - a obj thread, launch tasks, get a list of running tasks or kill a task.
 obj.users                     - manages users.
 obj.utils                     - utility module.

programming your own commands is easy, your can load modules with the -m option.
a command is a function with one argument, the event that was generated on the bot

::

 def mycommand(event):

     <<< your code here >>>

You can use event.reply() to send response back to the user.

OBJ has a "no-clause MIT license" that should be the most liberal license you can get at the year 2019.

  
    """,
    zip_safe=False,
    license='MIT',
    packages=["obj"],
    classifiers=['Development Status :: 3 - Alpha',
                 'Environment :: Console',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: MIT License',
                 'Operating System :: Unix',
                 'Programming Language :: Python',
                 'Topic :: Software Development :: Libraries :: Application Frameworks'
                ]
)
